/*
 * Copyright 2015, Yahoo Inc.
 * Copyrights licensed under the New BSD License.
 * See the accompanying LICENSE file for terms.
 */
import * as React from 'react';
import withIntl, { Provider } from './injectIntl';
import * as invariant_ from 'invariant';
// Since rollup cannot deal with namespace being a function,
// this is to interop with TypeScript since `invariant`
// does not export a default
// https://github.com/rollup/rollup/issues/1267
const invariant = invariant_;
import { createError, filterProps, DEFAULT_INTL_CONFIG, createDefaultFormatters, } from '../utils';
import { formatters } from '../format';
import areIntlLocalesSupported from 'intl-locales-supported';
import * as shallowEquals_ from 'shallow-equal/objects';
const shallowEquals = shallowEquals_;
const intlConfigPropNames = [
    'locale',
    'timeZone',
    'formats',
    'messages',
    'textComponent',
    'defaultLocale',
    'defaultFormats',
    'onError',
];
function getConfig(filteredProps) {
    let config = Object.assign({}, DEFAULT_INTL_CONFIG, filteredProps);
    // Apply default props. This must be applied last after the props have
    // been resolved and inherited from any <IntlProvider> in the ancestry.
    // This matches how React resolves `defaultProps`.
    for (const propName in DEFAULT_INTL_CONFIG) {
        if (config[propName] === undefined) {
            config[propName] =
                DEFAULT_INTL_CONFIG[propName];
        }
    }
    if (!config.locale || !areIntlLocalesSupported(config.locale)) {
        const { locale, defaultLocale, defaultFormats, onError } = config;
        if (typeof onError === 'function')
            onError(createError(`Missing locale data for locale: "${locale}". ` +
                `Using default locale: "${defaultLocale}" as fallback.`));
        // Since there's no registered locale data for `locale`, this will
        // fallback to the `defaultLocale` to make sure things can render.
        // The `messages` are overridden to the `defaultProps` empty object
        // to maintain referential equality across re-renders. It's assumed
        // each <FormattedMessage> contains a `defaultMessage` prop.
        config = Object.assign({}, config, { locale: defaultLocale, formats: defaultFormats, messages: DEFAULT_INTL_CONFIG.messages });
    }
    return config;
}
function getBoundFormatFns(config, state) {
    const formatterState = Object.assign({}, state.context.formatters);
    return {
        formatNumber: formatters.formatNumber.bind(undefined, config, formatterState),
        formatRelativeTime: formatters.formatRelativeTime.bind(undefined, config, formatterState),
        formatDate: formatters.formatDate.bind(undefined, config, formatterState),
        formatTime: formatters.formatTime.bind(undefined, config, formatterState),
        formatPlural: formatters.formatPlural.bind(undefined, config, formatterState),
        formatMessage: formatters.formatMessage.bind(undefined, config, formatterState),
        formatHTMLMessage: formatters.formatHTMLMessage.bind(undefined, config, formatterState),
    };
}
class IntlProvider extends React.PureComponent {
    constructor(props) {
        super(props);
        invariant(typeof Intl !== 'undefined', '[React Intl] The `Intl` APIs must be available in the runtime, ' +
            'and do not appear to be built-in. An `Intl` polyfill should be loaded.\n' +
            'See: http://formatjs.io/guides/runtime-environments/');
        const { intl: intlContext } = props;
        // Creating `Intl*` formatters is expensive. If there's a parent
        // `<IntlProvider>`, then its formatters will be used. Otherwise, this
        // memoize the `Intl*` constructors and cache them for the lifecycle of
        // this IntlProvider instance.
        const { formatters = createDefaultFormatters() } = intlContext || {};
        this.state = {
            context: Object.assign({}, intlContext, { formatters }),
        };
    }
    static getDerivedStateFromProps(nextProps, prevState) {
        const { intl: intlContext } = nextProps;
        // Build a whitelisted config object from `props`, defaults, and
        // `props.intl`, if an <IntlProvider> exists in the ancestry.
        const filteredProps = filterProps(nextProps, intlConfigPropNames, intlContext || {});
        if (!shallowEquals(filteredProps, prevState.filteredProps)) {
            const config = getConfig(filteredProps);
            const boundFormatFns = getBoundFormatFns(config, prevState);
            return {
                filteredProps,
                context: Object.assign({}, prevState.context, config, boundFormatFns),
            };
        }
        return null;
    }
    render() {
        return (React.createElement(Provider, { value: this.state.context }, this.props.children));
    }
}
export default withIntl(IntlProvider, { enforceContext: false }); // to be able to inherit values from parent providers
